#!/bin/bash

# standard P2J interactive client startup script

# defaults
srcname=$0
cfgdir=${srcname%/*}
up="../../../.."
debug=0
suspend="n"
prog="java"
dump="-XX:+HeapDumpOnOutOfMemoryError"
#dump="-XX:+HeapDumpOnOutOfMemoryError -XX:OnOutOfMemoryError='./stack_dump.sh %p'"
swing="client:driver:type=chui_swing client:chui:rows=24 client:chui:columns=80 client:chui:background=0x000000 client:chui:foreground=0xFFA500 client:chui:selection=0x0000FF client:chui:fontname=monospaced client:chui:fontsize=12"
host="net:server:host=localhost"
cfg=""
logdir=$P2J_LOG_DIR
log=""
srvr=""
test="n"
codepath=""
instance=0
portbase=3333
portstep=100

# help text
usage[0]="\nUsage: $0 [-dport] [-s] [-t] [-llogfile] [-ccfgfile] [-iinstance] [-2] [-zcodepath]\n"
usage[1]="Where:\n"
usage[2]="d \t= enable JVM debug mode, use the specified TCP port"
usage[3]="s \t= in JVM debug mode enable, suspend the JVM on startup (does not suspend by default)"
usage[4]="t \t= test mode (displays the command but doesn't execute)"
usage[5]="l \t= log file name in your home dir (all STDERR is redirected to this file, defaults to stderr.log)"
usage[6]="h \t= hostname or IP address for the server (defaults to localhost)"
usage[7]="i \t= server instance number (0..9, assigns the P2J server port, defaults to 0) to which to connect"
usage[8]="c \t= config file name (P2J bootstrap config file name, defaults to standard_client.xml)"
usage[9]="2 \t= enable C2 HotSpot compiler (server compiler for the JVM)"
# usage[10]="z \t= specify the path (with no trailing slash) to the location of the p2j.jar, libTerminal.so and libp2j.so"

# process options
while getopts ":d:stu2z:l:h:c:" opt; do
   case $opt in
      d  ) debug=$OPTARG ;;
      s  ) suspend="y" ;;
      t  ) prog="echo "$prog ; test="y" ;;
      l  ) log=$OPTARG ;;
      h  ) host="net:server:host="$OPTARG ;;
      i  ) instance=$OPTARG ;;
      c  ) cfg=$OPTARG ;; 
      2  ) srvr="-server" ;;
      u  ) up=$up"/"$up ;;
      z  ) codepath=$OPTARG ;;
      \? ) for i in "${usage[@]}"; do
              echo -e $i
           done
           exit 1 ;;
   esac
done
shift $(($OPTIND - 1))

# pass the remaining command line arguments directly to the java driver
# WARNING: any arguments with embedded spaces are not passed properly; for
#          example, if you pass this to the script:
#             client:cmd-line-option:param="\"\\\"some text here\\\"\""
#          you would expect the following to be passed to the java driver:
#             client:cmd-line-option:param="\"some text here\""
#          but that does not happen; when running the driver directly this would
#          work fine; via the script the result is that the driver sees add'l 
#          parameters, which will cause it to think it has add'l bootstrap config
#          files to load 
remainder="$@"
# final arg prep
sport=$(($portbase + $instance))
iport="net:server:insecure_port="$(($portbase + $instance + $portstep))
port="net:connection:secure=false net:server:secure_port=$sport $iport"

if [ $debug -ne 0 ]; then
   dtxt="-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,address=$debug,server=y,suspend=$suspend" 
else
   dtxt=""
fi

if [ $log"." == "." ]; then
   if [ $logdir"." == "." ]; then
      logdir="."
   fi
   log="${logdir}/client_$(whoami)_${$}.log"
fi   

if [ $codepath"." == "." ]; then
   codepath="$../lib"
fi   

if [ $cfg"." != "." ]; then
   cfg="$cfgdir/$cfg"
fi

lpath="-Djava.library.path=../lib/"
cpath="-classpath ../lib/p2j.jar:"

for each in ../lib/*.jar
do
   cpath=$cpath:$each
done

# run the P2J client
cmd="exec $prog $srvr $dump $dtxt $lpath $cpath com.goldencode.p2j.main.ClientDriver $host $port $swing $cfg $remainder"
if [ "$test" == "y" ]; then
   $cmd
else
   $cmd 2>$log
fi

